@@ -1,13 +1,13 @@
 package Net::DNS;
 
 #
-# $Id: DNS.pm 1562 2017-04-19 13:08:42Z willem $
+# $Id: DNS.pm 1570 2017-05-31 09:01:57Z willem $
 #
 require 5.006;
 our $VERSION;
-$VERSION = '1.09_01';
+$VERSION = '1.10_01';
 $VERSION = eval $VERSION;
-our $SVNVERSION = (qw$LastChangedRevision: 1562 $)[1];
+our $SVNVERSION = (qw$LastChangedRevision: 1570 $)[1];
 
 
 =head1 NAME
@@ -178,7 +178,7 @@ sub rr_add {
 
 sub rr_del {
 	my ( $domain, @etc ) = map split, @_;
-	my $rr = new Net::DNS::RR( scalar(@etc) ? @_ : ( name => $domain ) );
+	my $rr = new Net::DNS::RR( scalar(@etc) ? @_ : ( name => $domain, type => 'ANY' ) );
 	$rr->class( $rr->rdata ? 'NONE' : 'ANY' );
 	$rr->ttl(0);
 	return $rr;
@@ -442,8 +442,8 @@ date information to remain useful.
 
 rrsort() provides functionality to help you sort RR arrays. In most cases
 rrsort() will give you the answer that you want, but you can specify your
-own sorting method by using the Net::DNS::RR::FOO->set_rrsort_func() class
-method. See Net::DNS::RR for details.
+own sorting method by using the C<< Net::DNS::RR::FOO->set_rrsort_func() >>
+class method. See L<Net::DNS::RR> for details.
 
 =head2 rrsort()
 
@@ -492,11 +492,11 @@ See L<Net::DNS::Update> for an example of performing dynamic updates.
 
     use Net::DNS;
     my $res   = Net::DNS::Resolver->new;
-    my $reply = $res->search("host.example.com");
+    my $reply = $res->search("www.example.com", "A");
 
     if ($reply) {
 	foreach my $rr ($reply->answer) {
-	    print $rr->address, "\n" if $rr->type eq "A";
+	    print $rr->address, "\n" if $rr->can("address");
 	}
     } else {
 	warn "query failed: ", $res->errorstring, "\n";
@@ -510,7 +510,7 @@ See L<Net::DNS::Update> for an example of performing dynamic updates.
     my $reply = $res->query("example.com", "NS");
 
     if ($reply) {
-	foreach $rr (grep { $_->type eq 'NS' } $reply->answer) {
+	foreach $rr (grep { $_->type eq "NS" } $reply->answer) {
 	    print $rr->nsdname, "\n";
 	}
     } else {
@@ -527,7 +527,7 @@ See L<Net::DNS::Update> for an example of performing dynamic updates.
 
     if (@mx) {
 	foreach $rr (@mx) {
-	    print $rr->preference, " ", $rr->exchange, "\n";
+	    print $rr->preference, "\t", $rr->exchange, "\n";
 	}
     } else {
 	warn "Can not find MX records for $name: ", $res->errorstring, "\n";
@@ -541,9 +541,11 @@ See L<Net::DNS::Update> for an example of performing dynamic updates.
     my $reply = $res->query("example.com", "SOA");
 
     if ($reply) {
-	($reply->answer)[0]->print;
+	foreach my $rr ($reply->answer) {
+	    $rr->print;
+	}
     } else {
-	print "query failed: ", $res->errorstring, "\n";
+	warn "query failed: ", $res->errorstring, "\n";
     }
 
 
@@ -551,6 +553,7 @@ See L<Net::DNS::Update> for an example of performing dynamic updates.
 
     use Net::DNS;
     my $res  = Net::DNS::Resolver->new;
+    $res->tcp_timeout(20);
     $res->nameservers("ns.example.com");
 
     my @zone = $res->axfr("example.com");
@@ -559,11 +562,15 @@ See L<Net::DNS::Update> for an example of performing dynamic updates.
 	$rr->print;
     }
 
+    warn $res->errorstring if $res->errorstring;
+
 
 =head2 Perform a background query and print the reply.
 
     use Net::DNS;
     my $res    = Net::DNS::Resolver->new;
+    $res->udp_timeout(10);
+    $res->tcp_timeout(20);
     my $socket = $res->bgsend("host.example.com");
 
     while ( $res->bgbusy($socket) ) {
@@ -572,12 +579,16 @@ See L<Net::DNS::Update> for an example of performing dynamic updates.
     }
 
     my $packet = $res->bgread($socket);
-    $packet->print;
+    if ($packet) {
+	$packet->print;
+    } else {
+	warn "query failed: ", $res->errorstring, "\n";
+    }
 
 
 =head1 BUGS
 
-C<Net::DNS> is slow.
+Net::DNS is slow.
 
 For other items to be fixed, or if you discover a bug in this
 distribution please use the CPAN bug reporting system.
