@@ -1,9 +1,9 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1530 2017-01-27 10:40:37Z willem $
+# $Id: Base.pm 1569 2017-05-31 09:01:09Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1530 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1569 $)[1];
 
 
 #
@@ -40,9 +40,9 @@ use constant SOCKS => scalar eval 'require Config; $Config::Config{usesocks}';
 
 
 # Allow taint tests to be optimised away when appropriate.
-use constant UTIL  => defined eval 'require Scalar::Util';
 use constant UNCND => $] < 5.008;	## eval '${^TAINT}' breaks old compilers
-use constant TAINT => UTIL && ( UNCND || eval '${^TAINT}' );
+use constant TAINT => UNCND || eval '${^TAINT}';
+use constant TESTS => TAINT && defined eval 'require Scalar::Util';
 
 
 use strict;
@@ -55,14 +55,25 @@ use IO::Socket;
 use Net::DNS::RR;
 use Net::DNS::Packet;
 
-use constant INT16SZ  => 2;
 use constant PACKETSZ => 512;
 
 
+my $uname = eval {
+	local $ENV{PATH} = TAINT ? '/bin:/usr/bin' : $ENV{PATH};
+	local *HANDLE;
+	open( HANDLE, 'uname -n |' ) or die "uname: $!";
+	local $_ = <HANDLE>;
+	close(HANDLE) or die "$? $!";
+	chomp;
+	return $_;
+};
+
+
 #
 # Set up a closure to be our class data.
 #
 {
+	my ( $host, @domain ) = _untaint( split /[.]/, $uname, 2 );
 	my $defaults = bless {
 		nameserver4	=> ['127.0.0.1'],
 		nameserver6	=> ['::1'],
@@ -70,7 +81,7 @@ use constant PACKETSZ => 512;
 		srcaddr4	=> '0.0.0.0',
 		srcaddr6	=> '::',
 		srcport		=> 0,
-		searchlist	=> [],
+		searchlist	=> [@domain],
 		retrans		=> 5,
 		retry		=> 4,
 		usevc		=> ( SOCKS ? 1 : 0 ),
@@ -78,6 +89,7 @@ use constant PACKETSZ => 512;
 		recurse		=> 1,
 		defnames	=> 1,
 		dnsrch		=> 1,
+		ndots		=> 1,
 		debug		=> 0,
 		tcp_timeout	=> 120,
 		udp_timeout	=> 30,
@@ -100,8 +112,8 @@ use constant PACKETSZ => 512;
 
 # These are the attributes that the user may specify in the new() constructor.
 my %public_attr = (
-	map( {$_ => $_} keys %{&_defaults}, qw(domain nameserver nameservers prefer_v6 srcaddr) ),
-	map( {$_ => 0} qw(nameserver4 nameserver6 srcaddr4 srcaddr6) ),
+	map( ( $_ => $_ ), keys %{&_defaults}, qw(domain nameserver nameservers prefer_v6 srcaddr) ),
+	map( ( $_ => 0 ), qw(nameserver4 nameserver6 srcaddr4 srcaddr6) ),
 	);
 
 
@@ -115,10 +127,9 @@ sub new {
 	my $init = $initial;
 	$initial ||= bless {%$base}, $class;
 	if ( my $file = $args{config_file} ) {
-		$self = bless {%$initial}, $class;
-		$self->_read_config_file($file);		# user specified config
-		$self->nameserver( map { m/^(.*)$/; $1 } $self->nameserver );
-		$self->searchlist( map { m/^(.*)$/; $1 } $self->searchlist );
+		my $conf = bless {%$initial}, $class;
+		$conf->_read_config_file($file);		# user specified config
+		$self = bless {_untaint(%$conf)}, $class;
 		%$base = %$self unless $init;			# define default configuration
 
 	} elsif ($init) {
@@ -147,12 +158,25 @@ my %resolv_conf = (			## map traditional resolv.conf option names
 	timeout	 => 'retrans',
 	);
 
-my %env_option = (			## any resolver attribute except as listed below
+my %res_option = (			## any resolver attribute except as listed below
 	%public_attr,
 	%resolv_conf,
-	map { $_ => 0 } qw(nameserver nameservers domain searchlist),
+	map( ( $_ => $_ ), qw(nameserver nameservers domain searchlist) ),
 	);
 
+sub _option {
+	my ( $self, $name, @value ) = @_;
+	my $attribute = $res_option{lc $name} || return;
+	push @value, 1 unless scalar @value;
+	$self->$attribute(@value);
+}
+
+
+sub _untaint {
+	return TAINT ? map ref($_) ? [_untaint(@$_)] : do { /^(.*)$/; $1 }, @_ : @_;
+}
+
+
 sub _read_env {				## read resolver config environment variables
 	my $self = shift;
 
@@ -162,13 +186,8 @@ sub _read_env {				## read resolver config environment variables
 
 	$self->searchlist( map split, $ENV{RES_SEARCHLIST} ) if exists $ENV{RES_SEARCHLIST};
 
-	if ( exists $ENV{RES_OPTIONS} ) {
-		foreach ( map split, $ENV{RES_OPTIONS} ) {
-			my ( $name, $val ) = split( m/:/, $_, 2 );
-			my $attribute = $env_option{$name} || next;
-			$val = 1 unless defined $val;
-			$self->$attribute($val);
-		}
+	foreach ( map split, $ENV{RES_OPTIONS} || '' ) {
+		$self->_option( split m/:/ );
 	}
 }
 
@@ -177,11 +196,11 @@ sub _read_config_file {			## read resolver config file
 	my $self = shift;
 	my $file = shift;
 
-	my @ns;
-
 	local *FILE;
+	open( FILE, $file ) or croak "$file: $!";
 
-	open( FILE, $file ) or croak "Could not open $file: $!";
+	my @nameserver;
+	my @searchlist;
 
 	local $_;
 	while (<FILE>) {
@@ -189,18 +208,7 @@ sub _read_config_file {			## read resolver config file
 
 		/^nameserver/ && do {
 			my ( $keyword, @ip ) = grep defined, split;
-			push @ns, @ip;
-			next;
-		};
-
-		/^option/ && do {
-			my ( $keyword, @option ) = grep defined, split;
-			foreach (@option) {
-				my ( $name, $val ) = split( m/:/, $_, 2 );
-				my $attribute = $resolv_conf{$name} || next;
-				$val = 1 unless defined $val;
-				$self->$attribute($val);
-			}
+			push @nameserver, @ip;
 			next;
 		};
 
@@ -211,15 +219,23 @@ sub _read_config_file {			## read resolver config file
 		};
 
 		/^search/ && do {
-			my ( $keyword, @searchlist ) = grep defined, split;
-			$self->searchlist(@searchlist);
+			my ( $keyword, @domain ) = grep defined, split;
+			push @searchlist, @domain;
 			next;
 		};
+
+		/^option/ && do {
+			my ( $keyword, @option ) = grep defined, split;
+			foreach (@option) {
+				$self->_option( split m/:/ );
+			}
+		};
 	}
 
 	close(FILE);
 
-	$self->nameservers(@ns);
+	$self->nameservers(@nameserver) if @nameserver;
+	$self->searchlist(@searchlist)	if @searchlist;
 }
 
 
@@ -228,22 +244,20 @@ sub string {
 	$self = $self->_defaults unless ref($self);
 
 	my @nslist = $self->nameservers();
-	my $domain = $self->domain;
 	return <<END;
 ;; RESOLVER state:
-;; domain	= $domain
-;; searchlist	= @{$self->{searchlist}}
 ;; nameservers	= @nslist
+;; searchlist	= @{$self->{searchlist}}
+;; ndots	= $self->{ndots}
 ;; defnames	= $self->{defnames}	dnsrch		= $self->{dnsrch}
 ;; retrans	= $self->{retrans}	retry		= $self->{retry}
 ;; recurse	= $self->{recurse}	igntc		= $self->{igntc}
-;; usevc	= $self->{usevc}	port		= $self->{port}
 ;; tcp_timeout	= $self->{tcp_timeout}	persistent_tcp	= $self->{persistent_tcp}
 ;; udp_timeout	= $self->{udp_timeout}	persistent_udp	= $self->{persistent_udp}
+;; usevc	= $self->{usevc}	port		= $self->{port}
 ;; prefer_v4	= $self->{prefer_v4}	force_v4	= $self->{force_v4}
 ;; debug	= $self->{debug}	force_v6	= $self->{force_v6}
 END
-
 }
 
 
@@ -290,7 +304,7 @@ sub nameservers {
 
 		$self->errorstring( $defres->errorstring );
 
-		my %address = map { ( $_ => $_ ) } @iplist;	# tainted
+		my %address = map( ( $_ => $_ ), @iplist );	# tainted
 		my @unique = values %address;
 		carp "unresolvable name: $ns" unless @unique;
 		push @ipv4, grep _ipv4($_), @unique;
@@ -361,14 +375,15 @@ sub query {
 	my $self = shift;
 	my $name = shift || '.';
 
-	# resolve name containing no dots or colons by appending domain
-	my @sfix = $self->{defnames} && $name !~ m/[:.]/ ? $self->domain : ();
+	my @sfix;
+
+	if ( $self->{defnames} && ( ( $name =~ tr/././ ) < $self->{ndots} ) ) {
+		@sfix = $self->domain unless $name =~ m/:|\.\d*$/;
+	}
+
 	my $fqdn = join '.', $name, @sfix;
-
 	$self->_diag( 'query(', $fqdn, @_, ')' );
-
 	my $packet = $self->send( $fqdn, @_ ) || return;
-
 	return $packet->header->ancount ? $packet : undef;
 }
 
@@ -379,16 +394,14 @@ sub search {
 	return $self->query(@_) unless $self->{dnsrch};
 
 	my $name = shift || '.';
-	my @sfix = $self->searchlist;
-	my @list = $name =~ m/[.]/ ? ( undef, @sfix ) : ( @sfix, undef );
 
-	foreach my $suffix ( $name =~ m/:|\.\d*$/ ? undef : @list ) {
+	my @sfix = ( $name =~ m/:|\.\d*$/ ) ? () : @{$self->{searchlist}};
+	my ( $domain, @etc ) = ( $name =~ tr/././ ) < $self->{ndots} ? (@sfix) : ( undef, @sfix );
+
+	foreach my $suffix ( $domain, @etc ) {
 		my $fqname = $suffix ? join( '.', $name, $suffix ) : $name;
-
 		$self->_diag( 'search(', $fqname, @_, ')' );
-
 		my $packet = $self->send( $fqname, @_ ) || next;
-
 		return $packet->header->ancount ? $packet : next;
 	}
 
@@ -499,7 +512,7 @@ NAMESERVER: foreach my $ns (@ns) {
 
 			# handle failure to detect taint inside socket->send()
 			die 'Insecure dependency while running with -T switch'
-					if TAINT && Scalar::Util::tainted($dst_sockaddr);
+					if TESTS && Scalar::Util::tainted($dst_sockaddr);
 
 			$select->add($socket);
 
@@ -600,7 +613,7 @@ sub _bgsend_udp {
 
 		# handle failure to detect taint inside $socket->send()
 		die 'Insecure dependency while running with -T switch'
-				if TAINT && Scalar::Util::tainted($dst_sockaddr);
+				if TESTS && Scalar::Util::tainted($dst_sockaddr);
 
 		my $expire = time() + $self->{udp_timeout};
 		${*$socket}{net_dns_bg} = [$expire, $packet];
@@ -625,8 +638,8 @@ sub bgbusy {
 	}
 
 	return if $self->{igntc};
-	return unless $query;
 	return unless $handle->socktype() == SOCK_DGRAM;
+	return unless $query;					# SpamAssassin 3.4.1 workaround
 
 	my $ans = $self->_bgread($handle);
 	$$appendix[2] = [$ans];
@@ -634,8 +647,8 @@ sub bgbusy {
 	return unless $ans->header->tc;
 
 	$self->_diag('packet truncated: retrying using TCP');
-	my $tcp = $self->_bgsend_tcp( $query, $query->data );
-	return defined( $_[1] = $tcp ) if $tcp;
+	my $tcp = $self->_bgsend_tcp( $query, $query->data ) || return;
+	return defined( $_[1] = $tcp );
 }
 
 
@@ -685,7 +698,8 @@ sub _decode_reply {
 
 	my $header = $reply->header;
 	return unless $header->qr;
-	return unless $query;
+
+	return $reply unless $query;				# SpamAssassin 3.4.1 workaround
 	return ( $header->id != $query->header->id ) ? undef : $reply;
 }
 
@@ -694,6 +708,7 @@ sub axfr {				## zone transfer
 	eval {
 		my $self = shift;
 
+		# initialise iterator state vector
 		my ( $select, $verify, @rr, $soa ) = $self->_axfr_start(@_);
 
 		my $iterator = sub {	## iterate over RRs
@@ -716,7 +731,7 @@ sub axfr {				## zone transfer
 
 		return $iterator unless wantarray;
 
-		my @zone;					# assemble whole zone
+		my @zone;		## subvert iterator to assemble entire zone
 		while ( my $rr = $iterator->() ) {
 			push @zone, $rr, @rr;			# copy RRs en bloc
 			@rr = pop(@zone);			# leave last one in @rr
@@ -809,22 +824,22 @@ sub _axfr_next {
 sub _read_tcp {
 	my $socket = shift;
 
-	my $size_buf = '';
-	$socket->recv( $size_buf, INT16SZ );
-	my ($unread) = unpack 'n*', $size_buf;
+	my ( $s1, $s2 );
+	$socket->recv( $s1, 2 );				# one lump
+	$socket->recv( $s2, 2 - length $s1 );			# or two?
+	my $size = unpack 'n', pack( 'a*a*@2', $s1, $s2 );
 
 	my $buffer = '';
-	while ( $unread > 0 ) {
+	while ( ( my $read = length $buffer ) < $size ) {
 
 		# During some of my tests recv() returned undef even
-		# though there was no error.  Checking for the amount
+		# though there was no error.  Checking the amount
 		# of data read appears to work around that problem.
 
-		my $read_buf = '';
-		$socket->recv( $read_buf, $unread );
+		my $recv_buf;
+		$socket->recv( $recv_buf, $size - $read );
 
-		$unread -= length( $read_buf || last );
-		$buffer .= $read_buf;
+		$buffer .= $recv_buf || last;
 	}
 	return $buffer;
 }
@@ -1129,7 +1144,7 @@ sub AUTOLOAD {				## Default method
 	*{$AUTOLOAD} = sub {
 		my $self = shift;
 		$self = $self->_defaults unless ref($self);
-		$self->{$name} = shift if scalar @_;
+		$self->{$name} = shift || 0 if scalar @_;
 		return $self->{$name};
 	};
 
