@@ -1,10 +1,9 @@
 package Net::DNS::Resolver::UNIX;
 
 #
-# $Id: UNIX.pm 1406 2015-10-05 08:25:49Z willem $
+# $Id: UNIX.pm 1568 2017-05-27 06:40:20Z willem $
 #
-use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1406 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1568 $)[1];
 
 
 =head1 NAME
@@ -15,40 +14,25 @@ Net::DNS::Resolver::UNIX - Unix resolver class
 
 
 use strict;
+use warnings;
 use base qw(Net::DNS::Resolver::Base);
 
 
-my $resolv_conf = "/etc/resolv.conf";
-my $dotfile	= '.resolv.conf';
+my @config_file = grep -f $_ && -r _, '/etc/resolv.conf';
 
-my @resolv_conf = grep -f $_ && -r _, $resolv_conf;
-
-my @config_path;
-push( @config_path, $ENV{HOME} ) if exists $ENV{HOME};
-push( @config_path, '.' );
-
-my @config_file = grep -f $_ && -o _, map "$_/$dotfile", @config_path;
-
-
-sub _untaint {
-	map { m/^(.*)$/; $1 } grep defined, @_;
-}
+my $dotfile = '.resolv.conf';
+my @dotpath = grep defined, $ENV{HOME}, '.';
+my @dotfile = grep -f $_ && -o _, map "$_/$dotfile", @dotpath;
 
 
 sub _init {
 	my $defaults = shift->_defaults;
 
-	foreach (@resolv_conf) {
-		$defaults->_read_config_file($_);
-	}
+	map $defaults->_read_config_file($_), @config_file;
 
-	foreach my $attr (qw(nameservers searchlist)) {
-		$defaults->$attr( _untaint $defaults->$attr() );
-	}
+	%$defaults = Net::DNS::Resolver::Base::_untaint(%$defaults);
 
-	foreach (@config_file) {
-		$defaults->_read_config_file($_);
-	}
+	map $defaults->_read_config_file($_), @dotfile;
 
 	$defaults->_read_env;
 }
